@@ -1,11 +1,12 @@
 import json
 import copy
 import random
-from typing import List,Tuple,Dict
+from signal import raise_signal
+from typing import List, Tuple, Dict
 from urllib import parse
 from types import FunctionType
 from TDhelper.generic.classDocCfg import doc
-from TDhelper.network.http.REST_HTTP import GET, POST
+from network.http.REST_HTTP import GET, POST, PUT, DELETE, ContentType
 from TDhelper.Decorators.log import logging, logging_setup
 from TDhelper.generic.randGetValue import getValue
 
@@ -448,7 +449,13 @@ class RPC:
         self._apisTable = {}
         self._current_service = None
 
-    def RPC_ACCESS(self, entryPoint:List[str], access_token:str, method:str, **kwargs:'dict|Tuple'):
+    def RPC_ACCESS(
+        self,
+        entryPoint: List[str],
+        access_token: str,
+        method: str,
+        **kwargs: "dict|Tuple"
+    ):
         method = parse.quote(method)
         params = ""
         headers = {}
@@ -493,7 +500,7 @@ class RPC:
         else:
             return False, "can't found %s api." % service + method
 
-    def register(self, service: Dict, hosts: List, methods: List)-> Tuple:
+    def register(self, service: Dict, hosts: List, methods: List) -> Tuple:
         """
         注册服务
         - params:
@@ -835,34 +842,72 @@ class RPC:
                 m_uri = ret["uri"]
                 m_params = ret["params"]
                 m_method = ret["method"]
+                m_method= m_method.upper()
                 m_headers = {}
                 if "headers" in kwargs:
-                    if isinstance(kwargs['headers'],dict):
-                        m_headers = kwargs["headers"]
+                    if isinstance(kwargs["headers"], dict):
+                        for o in kwargs["headers"]:
+                            m_headers[
+                                o.lower().replace("http_", "").replace("_", "-")
+                            ] = kwargs["headers"][o]
                     else:
-                        return {"state": -1, "msg": "http request headers must is dict type."}
+                        return {
+                            "state": -1,
+                            "msg": "http request headers must is dict type.",
+                        }
                 m_headers["api-token"] = ret["secret"] if ret["secret"] else ""
                 params_data = ""
                 m_count = 0
                 for param in m_params:
                     if param["key"].lower() != "pk":
-                        if param["key"] in kwargs:
-                            params_data += (
-                                param["key"] + "=" + parse.quote(kwargs[param["key"]])
-                            )
+                        if param["key"].startswith("http_header_") or param[
+                            "key"
+                        ].startswith("HTTP_HEADER_"):
+                            if param["key"] not in m_headers:
+                                if param["key"] not in kwargs:
+                                    return {
+                                        "state": -1,
+                                        "msg": "not found parameter '%s'"
+                                        % (param["key"]),
+                                    }
+                                m_headers[
+                                    param["key"]
+                                    .replace("_", "-")
+                                    .replace("http-header-", "")
+                                    .replace("HTTP-HEADER-", "")
+                                ] = kwargs[param["key"]]
                         else:
-                            if param["defaultValue"]:
+                            if param["key"] in kwargs:
                                 params_data += (
                                     param["key"]
                                     + "="
-                                    + parse.quote(param["defaultValue"])
+                                    + (
+                                        parse.quote(kwargs[param["key"]])
+                                        if isinstance(kwargs[param["key"]], str)
+                                        else parse.quote(str(kwargs[param["key"]]))
+                                    )
                                 )
-                        m_count += 1
-                        if m_count < len(m_params):
-                            params_data += "&"
+                            else:
+                                if param["defaultValue"]:
+                                    params_data += (
+                                        param["key"]
+                                        + "="
+                                        + (
+                                            parse.quote(param["defaultValue"])
+                                            if isinstance(param["defaultValue"], str)
+                                            else parse.quote(str(param["defaultValue"]))
+                                        )
+                                    )
+                            m_count += 1
+                            if m_count < len(m_params):
+                                if len(params_data) > 0:
+                                    params_data += "&"
                     else:
                         m_uri = m_uri.replace(
-                            "{pk}", parse.quote(str(kwargs[param["key"]]))
+                            "{pk}",
+                            parse.quote(kwargs[param["key"]])
+                            if isinstance(kwargs[param["key"]], str)
+                            else parse.quote(str(kwargs[param["key"]])),
                         )
                 if params_data:
                     if m_method == u"GET" or m_method == u"DELETE":
@@ -871,7 +916,7 @@ class RPC:
                         else:
                             m_uri += "?" + params_data
             except Exception as e:
-                return {"state": -1, "msg": e}
+                return {"state": -1, "msg": e.args}
             try:
                 if m_method == u"GET":
                     logging.info("access api:(%s), method(GET)." % m_uri)
@@ -881,24 +926,32 @@ class RPC:
                             ret = json.loads(str(ret, encoding="utf-8"))
                             return {"state": ret["state"], "msg": ret["msg"]}
                         except Exception as e:
-                            return {"state": -1, "msg": e}
+                            logging.error(e.args)
+                            return {
+                                "state": state,
+                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
+                            }
                     else:
                         try:
                             if ret:
                                 ret = json.loads(str(ret, encoding="utf-8"))
                                 return {"state": ret["state"], "msg": ret["msg"]}
                             else:
-                                return {"state":state, "msg":"%s"%m_uri}
-                        except:
-                            return {"state": state, "msg": 'remote call "%s" error.'%m_uri}
+                                return {"state": state, "msg": "%s" % m_uri}
+                        except Exception as e:
+                            return {
+                                "state": state,
+                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
+                            }
                 elif m_method == u"POST":
                     logging.info(
                         "access api:(%s), method(POST),postdata:%s."
-                        % (m_uri, params_data)
+                        % (m_uri, kwargs["data"])
                     )
                     state, ret = POST(
                         m_uri,
-                        bytes(params_data, encoding="utf-8"),
+                        kwargs["data"],
+                        content_type=ContentType.JSON,
                         http_headers=m_headers,
                         time_out=15,
                     )
@@ -907,17 +960,24 @@ class RPC:
                             ret = json.loads(str(ret, encoding="utf-8"))
                             return {"state": ret["state"], "msg": ret["msg"]}
                         except Exception as e:
-                            logging.error(e)
-                            raise e
+                            logging.error(e.args)
+                            return {
+                                "state": state,
+                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
+                            }
                     else:
                         try:
                             if ret:
                                 ret = json.loads(str(ret, encoding="utf-8"))
                                 return {"state": ret["state"], "msg": ret["msg"]}
                             else:
-                                return {"state":state, "msg":"%s"%m_uri}
-                        except:
-                            return {"state": state, "msg": 'remote call "%s" error.'%m_uri}
+                                return {"state": state, "msg": "%s" % m_uri}
+                        except Exception as e:
+                            logging.error(e.args)
+                            return {
+                                "state": state,
+                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
+                            }
                 elif m_method == u"PUT":
                     logging.info(
                         "access api:(%s), method(PUT),postdata:%s."
@@ -929,10 +989,44 @@ class RPC:
                 elif m_method == u"DELETE":
                     # 还没有写DELETE方法
                     logging.info("access api:(%s), method(DELETE)" % m_uri)
-                    raise Exception("urllib delete 方法还没写.")
+                    if "data" in kwargs:
+                        state, ret = DELETE(
+                            m_uri,
+                            post_data=kwargs["data"],
+                            http_headers=m_headers,
+                            content_type=ContentType.JSON,
+                            time_out=15,
+                        )
+                    else:
+                        state, ret = DELETE(
+                            uri=m_uri, http_headers=m_headers, time_out=15
+                        )
+                    if state==200:
+                        try:
+                            ret= json.loads(str(ret,encoding='utf-8'))
+                            return {"state":ret["state"], "msg": ret["msg"]}
+                        except Exception as e:
+                            logging.error(e.args)
+                            return {
+                                "state": state,
+                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
+                            }
+                    else:
+                        try:
+                            if ret:
+                                ret = json.loads(str(ret, encoding="utf-8"))
+                                return {"state": ret["state"], "msg": ret["msg"]}
+                            else:
+                                return {"state": state, "msg": "%s" % m_uri}
+                        except Exception as e:
+                            logging.error(e.args)
+                            return {
+                                "state": state,
+                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
+                            }
                 else:
-                    raise Exception("restful method is error.")
+                    return {"state":-1, "msg":"restful method is error. POST|GET|DELET|PUT(unrealized)|OPTIONS(unrealized)|HEAD(unrealized)"}
             except Exception as e:
-                return {"state": -1, "msg": e}
+                return {"state": -1, "msg": e.args}
         else:
             return {"state": -1, "msg": ret}
